local GlobalAddonName, AGU = ...

if AZP == nil then AZP = {} end
AZP.GU = AGU

AZP.GU.VersionControl = {}
AZP.GU.OnLoad = {}
AZP.GU.OnEvent = {}

local VersionControl = AZP.GU.VersionControl
local OnLoad = AZP.GU.OnLoad
local OnEvent = AZP.GU.OnEvent

local AZPGUCoreVersion = 0.1
local dash = " - "
local name = "GameUtility"  .. dash .. "Core"
local nameFull = ("AzerPUG " .. name)
local promo = (nameFull .. dash ..  AZPGUCoreVersion)
local addonMain = LibStub("AceAddon-3.0"):NewAddon("GameUtility", "AceConsole-3.0")

local OptionsCorePanel
local addonVersions = AGU.addonVersions
local addonOutOfDateMessage = true

local GameUtilityLDB = LibStub("LibDataBroker-1.1"):NewDataObject("GameUtility", {
	type = "data source",
	text = "GameUtility",
	icon = "Interface\\Icons\\Inv_darkmoon_eye",
    OnClick = function() addonMain:ShowHideFrame() end
})
local icon = LibStub("LibDBIcon-1.0")

function addonMain:ShowHideFrame()
    if GameUtilityAddonFrame:IsShown() then
        GameUtilityAddonFrame:Hide()
        AGUFrameShown = false
    else
        GameUtilityAddonFrame:Show()
        AGUFrameShown = true
    end
end

function addonMain:OnInitialize()
    self.db = LibStub("AceDB-3.0"):New("GameUtilityLDB",
    {
        profile =
        {
            minimap =
            {
				hide = false,
			},
		},
	})
	icon:Register("GameUtility", GameUtilityLDB, self.db.profile.minimap)
	self:RegisterChatCommand("GameUtility icon", "MiniMapIconToggle")
end

function addonMain:OnLoad(self)
    addonMain:CreateMainFrame()
    OptionsCorePanel = CreateFrame("FRAME", "AZP-GU-OptionsCorePanel")
    OptionsCorePanel.name = "AzerPUG GameUtility"
    InterfaceOptions_AddCategory(OptionsCorePanel)

    local OptionsCoreHeader = OptionsCorePanel:CreateFontString("OptionsCoreHeader", "ARTWORK", "GameFontNormalHuge")
    OptionsCoreHeader:SetText(promo)
    OptionsCoreHeader:SetWidth(OptionsCorePanel:GetWidth())
    OptionsCoreHeader:SetHeight(OptionsCorePanel:GetHeight())
    OptionsCoreHeader:SetPoint("TOP", 0, -10)

    local OptionsCoreText = CreateFrame("Frame", "OptionsCoreText", OptionsCorePanel)
    OptionsCoreText:SetSize(500, 500)
    OptionsCoreText:SetPoint("TOP", 0, -50)
    OptionsCoreText.contentText = OptionsCoreText:CreateFontString("OptionsCoreText", "ARTWORK", "GameFontNormalLarge")
    OptionsCoreText.contentText:SetPoint("TOPLEFT")
    OptionsCoreText.contentText:SetText(
        "AzerPUG Links:\n" ..
        "Website: www.azerpug.com\n" ..
        "Discord: www.azerpug.com/discord\n" ..
        "Twitch: www.twitch.tv/azerpug\n"
    )

    local RepBarsSubPanel = CreateFrame("FRAME", "RepBarsSubPanel")
    RepBarsSubPanel.name = "RepBars"
    RepBarsSubPanel.parent = OptionsCorePanel
    InterfaceOptions_AddCategory(RepBarsSubPanel)

    local RepBarsSubPanelPHText = CreateFrame("Frame", "RepBarsSubPanelPHText", RepBarsSubPanel)
    RepBarsSubPanelPHText:SetSize(500, 500)
    RepBarsSubPanelPHText:SetPoint("TOP", 0, -50)
    RepBarsSubPanelPHText.contentText = RepBarsSubPanelPHText:CreateFontString("RepBarsSubPanelPHText", "ARTWORK", "GameFontNormalLarge")
    RepBarsSubPanelPHText.contentText:SetPoint("TOPLEFT")
    RepBarsSubPanelPHText.contentText:SetText(
        "RepBars Module provides the option to track several Factions at once.\n" ..
        "The factions can easily be selected within the standard GUI.\n"
    )

    local VersionControlFrame = CreateFrame("FRAME", "VersionControlFrame", UIParent)
    VersionControlFrame:SetPoint("TOP")
    VersionControlFrame:SetSize(200, 100)

    VersionControlFrame.texture = VersionControlFrame:CreateTexture()
    VersionControlFrame.texture:SetAllPoints(true)
    VersionControlFrame:EnableMouse(true)
    VersionControlFrame:SetMovable(true)
    VersionControlFrame.texture:SetColorTexture(0.5, 0.5, 0.5, 0.5)
    VersionControlFrame:RegisterForDrag("LeftButton")
    VersionControlFrame:SetScript("OnDragStart", VersionControlFrame.StartMoving)
    VersionControlFrame:SetScript("OnDragStop", VersionControlFrame.StopMovingOrSizing)

    VersionControlFrameCloseButton = CreateFrame("Button", "VersionControlFrameCloseButton", VersionControlFrame, "UIPanelButtonTemplate")
    VersionControlFrameCloseButton.contentText = VersionControlFrameCloseButton:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
    VersionControlFrameCloseButton.contentText:SetText("X")
    VersionControlFrameCloseButton:SetWidth("15")
    VersionControlFrameCloseButton:SetHeight("15")
    VersionControlFrameCloseButton.contentText:SetWidth("100")
    VersionControlFrameCloseButton.contentText:SetHeight("15")
    VersionControlFrameCloseButton:SetPoint("TOPRIGHT")
    VersionControlFrameCloseButton.contentText:SetPoint("CENTER", 0, -1)
    VersionControlFrameCloseButton:SetScript("OnClick", function() VersionControlFrame:Hide() end )

    local VersionControlFrameText = CreateFrame("Frame", "VersionControlFrameText", VersionControlFrame)
    VersionControlFrameText:SetSize(200, 100)
    VersionControlFrameText:SetPoint("TOP", 0, -5)
    VersionControlFrameText.contentText = VersionControlFrameText:CreateFontString("VersionControlFrameText", "ARTWORK", "GameFontNormalLarge")
    VersionControlFrameText.contentText:SetText("")
    VersionControlFrameText.contentText:SetPoint("TOP")

    VersionControlFrame:Hide()
end

function addonMain:CreateMainFrame()
    local GameUtilityAddonFrame = CreateFrame("FRAME", "GameUtilityAddonFrame", UIParent)
    GameUtilityAddonFrame:SetPoint("CENTER", 0, 0)
    GameUtilityAddonFrame:EnableMouse(true)
    GameUtilityAddonFrame:SetMovable(true)
    GameUtilityAddonFrame:RegisterForDrag("LeftButton")
    GameUtilityAddonFrame:SetScript("OnDragStart", GameUtilityAddonFrame.StartMoving)
    GameUtilityAddonFrame:SetScript("OnDragStop", GameUtilityAddonFrame.StopMovingOrSizing)
    GameUtilityAddonFrame:SetScript("OnEvent", function(...) addonMain:OnEvent(...) end)
    GameUtilityAddonFrame:RegisterEvent("PLAYER_ENTERING_WORLD")
    GameUtilityAddonFrame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
    GameUtilityAddonFrame:RegisterEvent("PLAYER_LOGIN")
    GameUtilityAddonFrame:RegisterEvent("ADDON_LOADED")
    GameUtilityAddonFrame:RegisterEvent("UPDATE_FACTION")
    GameUtilityAddonFrame:RegisterEvent("LFG_BONUS_FACTION_ID_UPDATED")
    GameUtilityAddonFrame:SetSize(325, 300)
    GameUtilityAddonFrame:SetBackdrop({
        bgFile = "Interface/Tooltips/UI-Tooltip-Background",
        edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
        edgeSize = 12,
        insets = { left = 1, right = 1, top = 1, bottom = 1 },
    })
    GameUtilityAddonFrame:SetBackdropColor(0.5, 0.5, 0.5, 0.75)

    
    local MainTitleFrame = CreateFrame("Frame", nil, GameUtilityAddonFrame)
    MainTitleFrame:SetHeight("20")
    MainTitleFrame:SetWidth(GameUtilityAddonFrame:GetWidth())
    MainTitleFrame:SetBackdrop({
        bgFile = "Interface/Tooltips/UI-Tooltip-Background",
        edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
        edgeSize = 12,
        insets = { left = 1, right = 1, top = 1, bottom = 1 },
    })
    MainTitleFrame:SetBackdropColor(0.3, 0.3, 0.3, 1)
    MainTitleFrame:SetPoint("TOP", 0, 0)
    MainTitleFrame.contentText = MainTitleFrame:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
    MainTitleFrame.contentText:SetWidth(MainTitleFrame:GetWidth())
    MainTitleFrame.contentText:SetHeight(MainTitleFrame:GetHeight())
    MainTitleFrame.contentText:SetPoint("CENTER", 0, -1)
    MainTitleFrame.contentText:SetText("\124cFF00FFFF" .. promo .. "\124r")

    IUAddonFrameCloseButton = CreateFrame("Button", "IUAddonFrameCloseButton", GameUtilityAddonFrame, "UIPanelButtonTemplate")
    IUAddonFrameCloseButton.contentText = IUAddonFrameCloseButton:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
    IUAddonFrameCloseButton.contentText:SetText("X")
    IUAddonFrameCloseButton:SetWidth("15")
    IUAddonFrameCloseButton:SetHeight("15")
    IUAddonFrameCloseButton.contentText:SetWidth("100")
    IUAddonFrameCloseButton.contentText:SetHeight("15")
    IUAddonFrameCloseButton:SetPoint("TOPRIGHT")
    IUAddonFrameCloseButton.contentText:SetPoint("CENTER", 0, -1)
    IUAddonFrameCloseButton:SetScript("OnClick", function() GameUtilityAddonFrame:Hide() end )
end

function addonMain:CoreVersionControl()
    if addonOutOfDateMessage == true then
        local tempText = "\124cFF00FFFFAzerPUG-InstanceUtility\nOut of date modules:\124r\n"
        local RepBarsVersion
        local ChatyThingsVersion
        local coreVersionUpdated = true

        if IsAddOnLoaded("AzerPUG-GameUtility-RepBars") then
            RepBarsVersion = VersionControl:RepBars()
            if RepBarsVersion < addonVersions["AZPGURepBarsVersion"] then
                tempText = tempText .. "\n\124cFFFF0000RepBars\124r"
                VersionControlFrame:Show()
            elseif RepBarsVersion > addonVersions["AZPGURepBarsVersion"] then
                coreVersionUpdated = false
            end
        end

        if IsAddOnLoaded("AzerPUG-GameUtility-ChatyThings") then
            ChatyThingsVersion = VersionControl:ChatyThings()
            if ChatyThingsVersion < addonVersions["AZPGUChatyThingsVersion"] then
                tempText = tempText .. "\n\124cFFFF0000ChatyThings\124r"
                VersionControlFrame:Show()
            elseif ChatyThingsVersion > addonVersions["AZPGUChatyThingsVersion"] then
                coreVersionUpdated = false
            end
        end

        if coreVersionUpdated == false then
            tempText = tempText .. "\n\124cFFFF0000Core\124r"
            VersionControlFrame:Show()
        end

        addonOutOfDateMessage = false
        VersionControlFrameText.contentText:SetText(tempText)
    end
end

function addonMain:OnEvent(self, event, ...)
    if event == "PLAYER_ENTERING_WORLD" then
        addonMain:CoreVersionControl()
        if AGUFrameShown == false then GameUtilityAddonFrame:Hide() end
    elseif event == "ADDON_LOADED" then
        local addonName = ...
        if addonName == "AzerPUG-GameUtility-RepBars" then
            OnLoad:RepBars()
            print("\124cffff00ffBlaBlaBla/124r")
        elseif addonName == "AzerPUG-GameUtility-ChatyThings" then
            OnLoad:ChatyThings()
        end
    end

    if event ~= "ADDON_LOADED" then
        if IsAddOnLoaded("AzerPUG-GameUtility-RepBars") then
            OnEvent:RepBars(event, ...)
        end
        if IsAddOnLoaded("AzerPUG-GameUtility-ChatyThings") then
            OnEvent:ChatyThings(event, ...)
        end
    end
end

addonMain:OnLoad()